<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;

use Ems\Services\AlteOcupacaoService;

class AlteOcupacaoController {

    /** @var OcorrenciaService */
    protected $alteOcupacaoService;
    
    public function __construct(EntityManager $entityManager, AlteOcupacaoService $alteOcupacaoService) {
        $this->entityManager        = $entityManager;
        $this->alteOcupacaoService   = $alteOcupacaoService;
    }
    
    public function getAlteocupacao(Request\Row $request, Response $response) {
        try {
            $params = $request->getRow();
            $nrvinculom = $params['NRVINCULOM'];
            $dtmescompetenc = $params['DTMESCOMPENTENC'] ?? null;
            
            $dados = $this->alteOcupacaoService->getAlteocupacao($nrvinculom, $dtmescompetenc);

            $response->addDataSet(new DataSet('getAlteocupacao', $dados));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function getAlteracaoCargo(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $dtInicio = $row['dtInicio'];
            $dtFinal = $row['dtFinal'];
            $nrCodIntEstrutLegal = $row['nrCodIntEstrutLegal'];
            
            $dados = $this->alteOcupacaoService->getAlteracaoCargo($dtInicio, $dtFinal, $nrCodIntEstrutLegal);

            $response->addDataSet(new DataSet('getAlteracaoCargo', $dados));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
}
